@@ -1,7 +1,8 @@
 #![deny(warnings)]
 use mt_dom::{diff::diff_with_functions, patch::*, *};
 
-type MyNode = Node<&'static str, &'static str, &'static str, &'static str>;
+type MyNode =
+    Node<&'static str, &'static str, &'static str, &'static str, &'static str>;
 
 #[test]
 fn force_replace() {
